/*
  Diamond Girl - Game where player collects diamonds.
  Copyright (C) 2005  Joni Yrjana
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


  Complete license can be found in the LICENSE file.
*/

#include "diamond_girl.h"

void draw_map(struct map * map, int top_x, int top_y, int width, int height, int map_x, int map_y, int map_fine_x, int map_fine_y,
	      int extra_life_anim, int morpher_is_on, int player_direction, int armour, int in_editor)
{ /* draw map */
  Uint32 c_black, c_illegal;

  {
    SDL_Rect r;

    r.x = top_x;
    r.y = top_y;
    r.w = width * 24;
    r.h = height * 24;
    SDL_SetClipRect(gfx(MAP_SCREEN), &r);
  }

  c_black = SDL_MapRGB(gfx(MAP_SCREEN)->format, 0, 0, 0);
  c_illegal = SDL_MapRGB(gfx(MAP_SCREEN)->format, 0x80, 0x40, 0x40);

  for(int y = 0 - 1; y < height; y++)
    for(int x = 0 - 1; x < width; x++)
      {
	SDL_Rect r;
	int dx, dy;
	char m;

	dx = top_x + x * 24 + map_fine_x;
	dy = top_y + y * 24 + map_fine_y;

	r.x = dx;
	r.y = dy;
	r.w = 24;
	r.h = 24;

	if(map_x + x >= 0 && map_x + x < map->width && 
	   map_y + y >= 0 && map_y + y < map->height)
	  m = map->data[map_x + x + (map_y + y) * map->width];
	else
	  m = MAP_ILLEGAL;

	switch((enum MAP_GLYPH) m)
	  {
	  case MAP_SMOKE2:
	    gfx_draw_glyph_frame(dx, dy, MAP_SMOKE, MOVE_NONE, 0);
	    break;
	  case MAP_SMOKE1:
	    gfx_draw_glyph_frame(dx, dy, MAP_SMOKE, MOVE_NONE, 1);
	    break;
	  case MAP_EXTRA_LIFE_ANIM:
	  case MAP_SMOKE:
	  case MAP_SCREEN:
	  case MAP_PLAYER_ARMOUR0:
	  case MAP_PLAYER_ARMOUR1:
	  case MAP_PLAYER_ARMOUR2:
	  case MAP_PLAYER_ARMOUR3:
	    break;
	  case MAP_BORDER:
	  case MAP_BRICK:
	    gfx_draw_glyph(dx, dy, MAP_BRICK, MOVE_NONE);
	    break;
	  case MAP_BRICK_MORPHER:
	    if(in_editor || morpher_is_on)
	      gfx_draw_glyph(dx, dy, MAP_BRICK_MORPHER, MOVE_NONE);
	    else
	      gfx_draw_glyph(dx, dy, MAP_BRICK, MOVE_NONE);
	    break;
	  case MAP_BRICK_EXPANDING:
	    if(in_editor)
	      gfx_draw_glyph(dx, dy, MAP_BRICK_EXPANDING, MOVE_NONE);
	    else
	      gfx_draw_glyph(dx, dy, MAP_BRICK, MOVE_NONE);
	    break;
	  case MAP_BRICK_UNBREAKABLE:
	    gfx_draw_glyph(dx, dy, MAP_BRICK_UNBREAKABLE, MOVE_NONE);
	    break;
	  case MAP_EMPTY:
	    if(extra_life_anim > 0)
	      gfx_draw_glyph(dx, dy, MAP_EXTRA_LIFE_ANIM, MOVE_NONE);
	    else
	      SDL_FillRect(gfx(MAP_SCREEN), &r, c_black);
	    break;
	  case MAP_ILLEGAL:
	    SDL_FillRect(gfx(MAP_SCREEN), &r, c_illegal);
	    break;
	  case MAP_SAND:
	    gfx_draw_glyph(dx, dy, MAP_SAND, MOVE_NONE);
	    break;
	  case MAP_BOULDER:
	  case MAP_BOULDER_FALLING:
	    gfx_draw_glyph(dx, dy, MAP_BOULDER, MOVE_NONE);
	    break;
	  case MAP_DIAMOND:
	  case MAP_DIAMOND_FALLING:
	    gfx_draw_glyph(dx, dy, MAP_DIAMOND, MOVE_NONE);
	    break;
	  case MAP_PLAYER:
	    {
	      enum MAP_GLYPH g;

	      if(armour == 0)
		g = MAP_PLAYER_ARMOUR0;
	      else if(armour == 1)
		g = MAP_PLAYER_ARMOUR1;
	      else if(armour == 2)
		g = MAP_PLAYER_ARMOUR2;
	      else //if(armour == 3)
		g = MAP_PLAYER_ARMOUR3;
	      gfx_draw_glyph(dx, dy, g, player_direction);
	    }
	    break;
	  case MAP_AMEBA:
	    gfx_draw_glyph(dx, dy, MAP_AMEBA, MOVE_NONE);
	    break;
	  case MAP_ENEMY1:
	    gfx_draw_glyph(dx, dy, MAP_ENEMY1, MOVE_NONE);
	    break;
	  case MAP_ENEMY2:
	    gfx_draw_glyph(dx, dy, MAP_ENEMY2, MOVE_NONE);
	    break;
	  case MAP_EXIT_LEVEL:
	    gfx_draw_glyph(dx, dy, MAP_EXIT_LEVEL, MOVE_NONE);
	    break;
	  }
      }
  SDL_SetClipRect(gfx(MAP_SCREEN), NULL);
}
